@@ -3,6 +3,10 @@
 from ml.app_logger import APP_LOGGER
 from ml.tools import remove_namespace, check_var_conflict
 from ml.ditto_feature import ditto_feature
+import sys
+import inspect
+from ml.parameters import Parameters
+from ml.entry import Entry
 from ml.thing import Thing
 from ml.ml40.roles.servives.service import Service
 from ml.ml40.roles.hmis.app import App
@@ -127,6 +131,8 @@ from ml.fml40.features.properties.values.documents.reports.passability_report im
 from ml.fml40.features.properties.values.documents.reports.soil_moisture_measurement import SoilMoistureMeasurement
 
 from ml.mml40.features.properties.values.Displacement import Displacement
+from ml.mml40.features.properties.values.GeometryProperties import GeometryProperties
+from ml.mml40.features.properties.values.LoadAlarm import LoadAlarm
 from ml.mml40.features.properties.values.MaterialProperties import MaterialProperties
 from ml.mml40.features.properties.values.Stretch import Stretch
 
@@ -176,6 +182,7 @@ from ml.fml40.features.functionalities.simulates_tree_growth import SimulatesTre
 from ml.fml40.features.functionalities.supports_felling import SupportsFelling
 from ml.fml40.features.functionalities.transports_logs import TransportsLogs
 
+from ml.mml40.features.functionalities.CantileverConfigure import CantileverConfigure
 from ml.mml40.features.functionalities.ProvidesDisplacementData import ProvidesDisplacementData
 from ml.mml40.features.functionalities.ProvidesForceData import ProvidesForceData
 from ml.mml40.features.functionalities.ProvidesStretchData import ProvidesStretchData
@@ -183,8 +190,6 @@ from ml.mml40.features.functionalities.ProvidesStretchData import ProvidesStretc
 # TODO: automatically get all classes in module
 DT_FACTORY = {}
 
-import sys, inspect
-
 clsmembers = inspect.getmembers(sys.modules[__name__], inspect.isclass)
 for member in clsmembers:
     DT_FACTORY[member[0]] = member[1]
@@ -221,6 +226,7 @@ def build_sub_features(feature_ins, feature):
                     build_sub_features(sub_f_instance, sub_f)
                 else:
                     setattr(sub_f_instance, key, sub_f[key])
+            sub_f_instance.parent = feature_ins
             feature_ins.subFeatures[sub_f_name] = sub_f_instance
 
 
@@ -236,9 +242,10 @@ def build_sub_thing(feature_ins, feature):
     """
     json_sub_things = feature.get("targets", [])
     for json_sub_thing in json_sub_things:
-        sub_thing_ref = create_thing(model={"attributes": json_sub_thing})
+        sub_thing_ref = create_thing(model_json={"attributes": json_sub_thing})
         sub_thing_name = json_sub_thing.get("name", None)
         feature_ins.targets[sub_thing_name] = sub_thing_ref
+        sub_thing_ref.parent = feature_ins
 
 
 def build(thing, model):
@@ -261,11 +268,13 @@ def build(thing, model):
     roles = attributes.get("roles", [])
     for role in roles:
         role_instance = build_role(role)
+        role_instance.parent = thing
         thing.roles[role.get("class")] = role_instance
 
     json_features = attributes.get("features", [])
     for feature in json_features:
         feature_ins = build_feature(feature=feature)
+        feature_ins.parent = thing
         thing.features[feature.get("class")] = feature_ins
 
     if ditto_features is not None:
@@ -341,6 +350,21 @@ def add_function_impl_obj(thing, impl_obj, feature_name, **kwargs):
 
     """
     feature = thing.features.get(feature_name, None)
+    if feature is None:
+        ### Check if it is a feature of a subthing
+        if thing.features.get("ml40::Composite", None) is not None:
+            subthings = thing.features["ml40::Composite"].targets
+            for i in subthings.keys():
+                feature = subthings[i].features.get(feature_name, None)
+                if feature is not None:
+                    break
+    if feature is None:
+        if thing.features.get("ml40::Shared", None) is not None:
+            subthings = thing.features["ml40::Shared"].targets
+            for i in subthings.keys():
+                feature = subthings[i].features.get(feature_name, None)
+                if feature is not None:
+                    break
     if feature is None:
         APP_LOGGER.critical(
             "Functionality %s is not one of the build-in functionalities" % feature_name
@@ -352,7 +376,40 @@ def add_function_impl_obj(thing, impl_obj, feature_name, **kwargs):
         thing.features[feature_name] = impl_ins
 
 
-def create_thing(model, grant_type="password",
+def create_thing(
+        model_json,
+        oauth2_secret="",
+        grant_type="client_credentials",
+        username=None,
+        password=None,
+        is_repository=False,
+        is_broker=False,
+        is_broker_rest=False,
+        parameters=Parameters()
+):
+    attributes = model_json.get("attributes")
+    if attributes is None:
+        APP_LOGGER.error("Attributes are empty")
+
+    identifier = model_json.get("thingId", "")
+    name = attributes.get("name", "")
+    APP_LOGGER.info("Build digital twin {} with id {}".format(name, identifier))
+    entry_ref = Entry(identifier, name)
+    build(entry_ref, model_json)
+    return Thing(
+        entry=entry_ref,
+        oauth2_secret=oauth2_secret,
+        grant_type=grant_type,
+        username=username,
+        password=password,
+        is_repository=is_repository,
+        is_broker=is_broker,
+        is_broker_rest=is_broker_rest,
+        parameters=parameters
+    )
+
+
+def _create_thing(model, grant_type="password",
                  secret="", username=None, password=None,
                  is_broker_rest=False, is_broker=False, is_repo=False, is_stanford2010=False,
                  is_stanford2010_sync=False,
@@ -385,7 +442,8 @@ def create_thing(model, grant_type="password",
     attributes = model.get("attributes", None)
     if attributes is None:
         sys.exit("attributes are none")
-    thing_type = remove_namespace(attributes.get("class", ""))
+    thing_type = attributes.get("class", "")
+    thing_type = remove_namespace(thing_type)
     thing_name = attributes.get("name", "")
     APP_LOGGER.info("Build digital twin {} with id {}".format(thing_name, model.get("thingId", "")))
 
