@@ -5,7 +5,7 @@ class AcceptsShieldCommands(Functionality):
     """This functionality signalizes that the thing's shield can be moved remotely
     via S3I-B messages ."""
 
-    def __init__(self, name="", identifier=""):
+    def __init__(self, namespace="fml40", name="", identifier="", parent=None):
         """Initializes the object.
 
         :param name:  Object name
@@ -13,8 +13,8 @@ class AcceptsShieldCommands(Functionality):
 
         """
         super().__init__(
-            name=name,
-            identifier=identifier)
+            namespace=namespace, name=name, identifier=identifier, parent=parent
+        )
 
     def shieldDown(self, speed):
         """roll down the winch with specified speed
@@ -32,4 +32,3 @@ class AcceptsShieldCommands(Functionality):
         :type speed: float
         """
         print("shield moves up with the speed {}".format(speed))
-
