@@ -2,10 +2,10 @@ from ml.ml40.features.properties.values.value import Value
 
 
 class TimeSlot(Value):
-    def __init__(self, name="", identifier=""):
+    def __init__(self, namespace="ml40", name="", identifier="", parent=None):
         super().__init__(
-            name=name,
-            identifier=identifier)
+            namespace=namespace, name=name, identifier=identifier, parent=parent
+        )
         self.end = None
         self.start = None
         self.__json_out = dict()
