@@ -1,4 +1,5 @@
 import asyncio
+import sys
 
 import s3i
 from s3i.identity_provider import IdentityProvider, TokenType
@@ -290,24 +291,26 @@ class Thing:
             ))
         except S3IDittoError as err:
             APP_LOGGER.error("[S3I]: {}".format(err))
-        if frequency != 0:
-            self.loop.call_later(
-                1 / frequency,
-                self.__recursively_update_directory,
-                frequency
-            )
+        finally:
+            if frequency != 0:
+                self.loop.call_later(
+                    1 / frequency,
+                    self.__recursively_update_directory,
+                    frequency
+                )
 
     def __recursively_update_repository(self, frequency):
         try:
             self.repo.updateThingIDBased(thingID=self.identifier, payload=self.__entry.refresh_repository_entry())
         except S3IDittoError as err:
             APP_LOGGER.error("[S3I]: {}".format(err))
-        if frequency != 0:
-            self.loop.call_later(
-                1 / frequency,
-                self.__recursively_update_repository,
-                frequency
-            )
+        finally:
+            if frequency != 0:
+                self.loop.call_later(
+                    1 / frequency,
+                    self.__recursively_update_repository,
+                    frequency
+                )
 
     def __recursively_update_dt_json(self, frequency):
         self.__entry.refresh_dt_json()
@@ -607,10 +610,16 @@ class Thing:
                         results=result,
                         message_id="s3i:{}".format(uuid.uuid4())
                     )
-
-        res = self.broker.send(
-            endpoint=body_json.get("replyToEndpoint", None),
-            msg=service_reply.base_msg)
+        if sys.getsizeof(service_reply.base_msg["results"]) > self.parameters.broker_msg_compress_threshold:
+            service_reply.compress(msg_json=service_reply.base_msg, level=6)
+            res = self.broker.send(
+                endpoint=body_json.get("replyToEndpoint", None),
+                msg=service_reply.gzip_msg
+            )
+        else:
+            res = self.broker.send(
+                endpoint=body_json.get("replyToEndpoint", None),
+                msg=service_reply.base_msg)
 
         if res:
             APP_LOGGER.info("[S3I]: Sending serviceReply back")
